%% Interpolate image
%
% In Matlab, (x,y) are:
%
%      j               x
%   o------>        o------>
%   |               |
% i |  . I(i,j)   y |  . I(x,y)
%   |               |
%   v               v
%

function I = iminterpolate(I,sx,sy,mode)

    if size(size(I),2)==3; I = iminterpolate_multichannel(I,sx,sy,mode); return; end;
    
    if nargin<4; mode = 'linear'; end;
    
    % Find update points on moving image
    nx = size(I,2); ny = size(I,1);
    [y,x] = ndgrid(1:ny, 1:nx); % coordinate image
    x_prime = x + sx; % updated x values (1st dim, rows)
    y_prime = y + sy; % updated y values (2nd dim, cols)
    
    % intensity range
    if ~isequal(mode,'nearest');
        min_i = min(I(I>0));
        max_i = max(I(:));
    end
    
    % Interpolate updated image
    I = interpn(y,x,I,y_prime,x_prime,mode,0); % moving image intensities at updated points
    
    % Fix mask and intensity range
    if ~isequal(mode,'nearest');
        eps = 1e-2;
        I(I<(min_i+eps) & I>0) = min_i;
        I(I>(max_i-eps))       = max_i;
    end
    
end

%% Interpolate image
function I = iminterpolate_multichannel(I,sx,sy,mode)

    if nargin<4; mode = 'linear'; end;
    
    nchannels = size(I,3);

    for i=1:nchannels
        Ip(:,:,i) = iminterpolate(I(:,:,i),sx,sy,mode);
    end
    
    I = Ip;
    
end

