function jh = jointhist(A,B,nbins)
% jh = jointhist(A,B,nbins)
% Returns the Joint Histogram of the grayscale images
% A,B.
% They are assumed to be standardised, as in:
%
% * same dimensions and coordinate systems
%
% * grayscale, meaning intensities are doubles from 0 to 1 (mat2gray)
%   mat2gray(B, [0 max(B(:))])
%
% INPUT:
%  A,B: images
%  nbins: optional: number of desired bins to use in histogram (default: 64)
%
% OUTPUT:
%  jh: the joint histogram (if no output it gets displayed as a grayscale image)
%
% This is just an auxiliary function, similar code is used inside NMI.
%
% 
% EXAMPLE
% load mri;
%
% D = squeeze(D);
% c = jointhist(D(:,:,1),D(:,:,2))

if nargin < 3; nbins = 63; end;  % default 

A = (A-min(A(:)))/range(A(:));
B = (B-min(B(:)))/range(B(:));

%% Bins
% grayscale intensities go from 0 to 1
% thus nbins*intensities go from 0 to nbins, rounded 0,1,...,nbins-1
a = round((nbins-1)*A(:));
b = round((nbins-1)*B(:));

%% Joint Histogram
%
% * sparse(n,m,s) puts the value s(k) at index n(k),m(k) in a sparse matrix.
%
% * if s is a number instead of a vector, it puts that number everywhere
% 
% * if a pair of indices come multiple times, the values get added.
%
% That's exactly what we want for a joint histogram: count the number of
% times intensity a and b appear at the same place.
jh = sparse(a+1,b+1,1); % matlab indexing is 1-based

%% Display
if nargout == 0
    imagesc(full(jh)); axis equal; axis off
end
