%% Register two images
%  Changed: Dec 21st, 2011
%
function [Mp,sx,sy,sx_inv,sy_inv,vx,vy] = register(F,M,opt)

    if ~isfield(opt, 'niter');            opt.niter = 5;            end;
    if ~isfield(opt, 'sigma_diffusion');  opt.sigma_diffusion = 1;  end;
    if ~isfield(opt, 'sigma_fluid');      opt.sigma_fluid = 1;      end;
    if ~isfield(opt, 'sigma_i');          opt.sigma_i = 1;          end;
    if ~isfield(opt, 'sigma_x');          opt.sigma_x = 1;          end;
    if ~isfield(opt, 'id_fixed');         opt.id_fixed = 'fixed';   end;
    if ~isfield(opt, 'id_moving');        opt.id_moving='id_moving';end;
    if ~isfield(opt, 'descend');          opt.descend = 0;          end;
    if ~isfield(opt, 'do_plotenergy');    opt.do_plotenergy = 0;    end;
    if ~isfield(opt, 'do_avi');           opt.do_avi = 0;           end;
    if ~isfield(opt, 'do_print');         opt.do_print = 0;         end;
    if ~isfield(opt, 'do_display');       opt.do_display = 1;       end;
    
    if opt.do_avi
        if ~isfield(opt, 'aviobj')
            opt.aviobj = avifile('spectral-demons.avi','compression','None', 'fps',10);
            opt.do_closeavi = 1; % create and close avi file here
        end
        if ~isfield(opt, 'do_closeavi'); opt.do_closeavi = 0; end;
        global aviobj;
        aviobj = opt.aviobj;
    end;

    mode = 'nearest';
    %mode = 'linear';
    
    %% Piggyback image
    piggyback_scale = 1.2;
    [F,lim] = piggyback(F,piggyback_scale);
    [M,lim] = piggyback(M,piggyback_scale);
    
    %% T is the deformation from M to F
    vx = zeros(size(M)); % velocity field
    vy = zeros(size(M));
    if isfield(opt,'vx') && isfield(opt,'vy')
        vx = piggyback(opt.vx,piggyback_scale);
        vy = piggyback(opt.vy,piggyback_scale);
    end
    e_min = 1e+100;      % Minimal energy
    
    %% Iterate update fields
    for iter=1:opt.niter

        % Find update
        [sx,sy] = expfield(vx,vy);
        [ux,uy] = findupdate(F,M,sx,sy,opt.sigma_i,opt.sigma_x,mode);

        % Regularize update
        ux    = imgaussian(ux,opt.sigma_fluid);
        uy    = imgaussian(uy,opt.sigma_fluid);

        % Compute step (max half a pixel)
        step  = 1;
        
        % Update correspondence (demons) - additive
        vx = vx + step*ux;
        vy = vy + step*uy;

        % Regularize deformation
        vx = imgaussian(vx,opt.sigma_diffusion);
        vy = imgaussian(vy,opt.sigma_diffusion);

        % Compute energy
        e      = energy(F,M,vx,vy,opt.sigma_i,opt.sigma_x);
        disp(['Iteration: ' num2str(iter) ' - ' 'energy: ' num2str(e)]);
        if e<e_min
            vx_min = vx; vy_min = vy;
            e_min  = e;
        else
            if opt.descend;
                break;
            end
        end

        if opt.do_display
            
            % display deformation
            [sx,sy] = expfield(vx,vy);
            subplot(2,4,7); showvector(ux,uy,4,0,lim); title('Update');
            subplot(2,4,8); showvector(sx,sy,4,0,lim); title('Transformation');
            drawnow;
            
            % Display registration
            Mp     = iminterpolate(M,sx,sy,mode);
            diff   = abs(F-Mp);
            if opt.do_print
                %clf; showimage(F+4*Mp,'', 'lim',lim,'caxis',[0 5]); printfigure(['images/demons-iter-' num2str(iter)],4,4);
                clf; imagesc(F+4*Mp); daspect([1 1 1]); axis off; axis([lim(3)-0.5 lim(4)+0.5 lim(1)-0.5 lim(2)+0.5]); printfigure(['images/demons-iter-' num2str(iter)],4,4);
            end
            showimage(F,'Fixed', M,'Moving', Mp,'Warped', 0.5*F+0.5*Mp,'Sum', diff,'Diff', 'lim',lim,'nbrows',2); drawnow;

            
        end
        
        % Plot energy
        if opt.do_plotenergy
            plot_e(iter) = e;
            subplot(2,2,3)
            hold on;
            plot(1:iter,plot_e,'r-'); xlim([0 opt.niter]);
            xlabel('Iteration'); ylabel('Energy');
            hold off;
            drawnow
        end
        if opt.do_avi; aviobj = addframe(aviobj,getframe(gcf)); end;
        
    end
    
    % Get best Transformation
    %vx = vx_min; vy = vy_min;
    [sx,sy]         = expfield(+vx,+vy);
    [sx_inv,sy_inv] = expfield(-vx,-vy);
    
    %% Transform moving image
    Mp = iminterpolate(M,sx,sy);
    
    %% Unpiggyback
    Mp     = Mp(lim(1):lim(2),lim(3):lim(4));
    sx     = sx(lim(1):lim(2),lim(3):lim(4));
    sy     = sy(lim(1):lim(2),lim(3):lim(4));
    sx_inv = sx_inv(lim(1):lim(2),lim(3):lim(4));
    sy_inv = sy_inv(lim(1):lim(2),lim(3):lim(4));
    vx     = vx(lim(1):lim(2),lim(3):lim(4));
    vy     = vy(lim(1):lim(2),lim(3):lim(4));

    if opt.do_avi && opt.do_closeavi
        aviobj = close(aviobj);
    end
    
end

%% Get energy
function e = energy(F,M,vx,vy,sigma_i,sigma_x)

    % Intensity difference
    [sx,sy] = expfield(vx,vy);
    Mp     = iminterpolate(M,sx,sy);
    diff2  = (F-Mp).^2;
    area   = numel(M);
    
    % Three energy components
    e_sim  = sum(diff2(:)) / area;
    %e_dist = sum((cx(:)-sx(:)).^2 + (cy(:)-sy(:)).^2) / area;
    e_reg  = sum(sx(:).^2 + sy(:).^2) / area;
    
    % Total energy
    e      = e_sim + (sigma_i^2/sigma_x^2) * e_reg;
    e = e_sim;

end

