%% Register two images
%  Changed: Dec 21st, 2011
%
function [Mp, sx,sy, sx_inv,sy_inv, vx,vy, stats] = sym_register(F,M,opt)

    if ~isfield(opt, 'niter');            opt.niter = 5;            end;
    if ~isfield(opt, 'sigma_diffusion');  opt.sigma_diffusion = 1;  end;
    if ~isfield(opt, 'sigma_fluid');      opt.sigma_fluid = 1;      end;
    if ~isfield(opt, 'sigma_i');          opt.sigma_i = 1;          end;
    if ~isfield(opt, 'sigma_x');          opt.sigma_x = 1;          end;
    if ~isfield(opt, 'id_fixed');         opt.id_fixed = 'fixed';   end;
    if ~isfield(opt, 'id_moving');        opt.id_moving='id_moving';end;
    if ~isfield(opt, 'useminimum');       opt.useminimum = 1;       end;
    if ~isfield(opt, 'descend');          opt.descend = 0;          end;
    if ~isfield(opt, 'do_plotenergy');    opt.do_plotenergy = 0;    end;
    if ~isfield(opt, 'do_avi');           opt.do_avi = 0;           end;
    if ~isfield(opt, 'do_print');         opt.do_print = 0;         end;
    if ~isfield(opt, 'do_display');       opt.do_display = 1;       end;
    
    if opt.do_avi
        if ~isfield(opt, 'aviobj')
            opt.aviobj = avifile('demons.avi','compression','None', 'fps',10);
            opt.do_closeavi = 1; % create and close avi file here
        end
        if ~isfield(opt, 'do_closeavi'); opt.do_closeavi = 0; end;
        global aviobj;
        aviobj = opt.aviobj;
    end;

    %mode = 'nearest';
    mode = 'linear';
    
    %% Piggyback image
    piggyback_scale = 1.2;
    [F,lim] = piggyback(F,piggyback_scale);
    [M,lim] = piggyback(M,piggyback_scale);
    
    %% T is the deformation from M to F
    vx = zeros(size(M)); % velocity field
    vy = zeros(size(M));
    if isfield(opt,'vx') && isfield(opt,'vy')
        vx = piggyback(opt.vx,piggyback_scale);
        vy = piggyback(opt.vy,piggyback_scale);
    end

    e_min = energy(F,M,vx,vy,opt.sigma_i,opt.sigma_x);
    vx_min = vx; vy_min = vy;
    convergence_cnt = 0;
    disp(['Iteration: 0 - energy: ' num2str(e_min)]);

    %% Iterate update fields
    for iter=1:opt.niter

        % Find update
        [sx_fm,sy_fm] = expfield(+vx,+vy);
        [sx_mf,sy_mf] = expfield(-vx,-vy);
        [ux_fm,uy_fm] = findupdate(F,M,sx_fm,sy_fm,opt.sigma_i,opt.sigma_x,mode);
        [ux_mf,uy_mf] = findupdate(M,F,sx_mf,sy_mf,opt.sigma_i,opt.sigma_x,mode);
        
        % Average updates
        ux = (ux_fm - ux_mf)/2;
        uy = (uy_fm - uy_mf)/2;

        % Regularize update
        ux    = imgaussian(ux,opt.sigma_fluid);
        uy    = imgaussian(uy,opt.sigma_fluid);

        % Compute step (max half a pixel)
        step  = opt.sigma_x;
        
        % Update correspondence (demons) - additive
        vx = vx + step*ux;
        vy = vy + step*uy;

        % Regularize deformation
        vx = imgaussian(vx,opt.sigma_diffusion);
        vy = imgaussian(vy,opt.sigma_diffusion);

        % Compute energy
        e      = energy(F,M,vx,vy,opt.sigma_i,opt.sigma_x,lim);
        disp(['Iteration: ' num2str(iter) ' - ' 'energy: ' num2str(e)]);
        if e<e_min
            vx_min = vx; vy_min = vy;
            e_min  = e;
            convergence_cnt = 0;
        else
            if opt.descend;
                convergence_cnt = convergence_cnt + 1;
                if convergence_cnt>3
                    break;
                end
            end
        end
        stats.energy(iter) = e;

        if opt.do_display
            
            % display deformation
            [sx,sy] = expfield(vx,vy);
            subplot(2,4,7); showvector(ux,uy,4,0,lim); title('Update');
            subplot(2,4,8); showvector(sx,sy,4,0,lim); title('Transformation');
            drawnow;
            
            % Display registration
            Mp     = iminterpolate(M,sx,sy,mode);
            diff   = abs(F-Mp);
            if opt.do_print
                clf; showimage(F+4*Mp,'', 'lim',lim,'caxis',[0 5]); printfigure(['images/spectral-fixed-' opt.id_fixed '-moving-' opt.id_moving '-demons-iter-' num2str(iter)],3,3);
                %clf; showimage(F+4*Mp,'', 'lim',lim,'caxis',[0 5]); printfigure(['images/demons-iter-' num2str(iter)],4,4);
                %clf; imagesc(F+4*Mp); daspect([1 1 1]); axis off; axis([lim(3)-0.5 lim(4)+0.5 lim(1)-0.5 lim(2)+0.5]); printfigure(['images/demons-iter-' num2str(iter)],4,4);
            end
            showimage(F,'Fixed', M,'Moving', Mp,'Warped', diff,'Diff', 'lim',lim,'nbrows',2); drawnow;

            
            % Plot energy
            if opt.do_plotenergy
                subplot(2,2,3)
                hold on;
                plot(1:iter,stats.energy,'r-'); xlim([0 opt.niter]);
                xlabel('Iteration'); ylabel('Energy');
                hold off;
                drawnow
            end
            
            %% Show figure with arrows (for eccv paper)
            if opt.do_display
                if 0%iter==1 || iter==2 || iter==5 || iter==15
                    clf;imfuse(F,Mp); axis([15 82 20 87]+9);                          printfigure(['images/demons-iter-' num2str(iter)],2,2,0);
                    hold on; showvector(ux,uy,5,1); hold off; axis([15 82 20 87]+9);  printfigure(['images/demons-iter-' num2str(iter) '-arrows'],2,2,1);
                end
            end
        end
        
        if opt.do_avi
            [sx_fm,sy_fm] = expfield(+vx,+vy);
            Mp  = iminterpolate(M,  sx_fm,sy_fm, mode);

            gcf_old = gcf; figure(99); set(gcf,'color','white');
            subplot(2,4,4); showimagesc(Mp,'+ Demons (Moving)', 'lim',lim);
            %colormap gray; clf; subplot('position',[0 0 1 1]);
            %imagesc(Mp); daspect([1 1 1]); axis(ceil([43 170 43 170]/1)); axis off;
            %imagesc(Mp); daspect([1 1 1]); axis(ceil([59 59 63 63]/1)+ceil([0 36 0 36]/1)); axis off; % hat right
            %imagesc(Mp); daspect([1 1 1]); axis(ceil([119 119 75 75]/2)+ceil([0 36 0 36]/2)); axis off; % hat right
            drawnow;
            for f=1:1; aviobj = addframe(aviobj,getframe(gcf)); end;
            figure(gcf_old);
        end;
        
    end
    
    % Get best Transformation
    if opt.useminimum
        vx = vx_min;
        vy = vy_min;
    end
    [sx,sy]         = expfield(+vx,+vy);
    [sx_inv,sy_inv] = expfield(-vx,-vy);
    
    %% Transform moving image
    Mp = iminterpolate(M,sx,sy);
    
    %% Unpiggyback
    Mp     = Mp(lim(1):lim(2),lim(3):lim(4));
    sx     = sx(lim(1):lim(2),lim(3):lim(4));
    sy     = sy(lim(1):lim(2),lim(3):lim(4));
    sx_inv = sx_inv(lim(1):lim(2),lim(3):lim(4));
    sy_inv = sy_inv(lim(1):lim(2),lim(3):lim(4));
    vx     = vx(lim(1):lim(2),lim(3):lim(4));
    vy     = vy(lim(1):lim(2),lim(3):lim(4));

    if opt.do_avi && opt.do_closeavi
        aviobj = close(aviobj);
    end
    
end

%% Get energy
function e = energy(F,M,vx,vy,sigma_i,sigma_x,lim)

    if nargin<5; sigma_i = 1; end;
    if nargin<6; sigma_x = 1; end;
    if nargin<7; lim = [1 size(F,1) 1 size(F,2)]; end;
    
    % use real image size
    F = F(lim(1):lim(2),lim(3):lim(4));
    M = M(lim(1):lim(2),lim(3):lim(4));
    vx = vx(lim(1):lim(2),lim(3):lim(4));
    vy = vy(lim(1):lim(2),lim(3):lim(4));
        
    % Intensity difference
    [sx,sy] = expfield(vx,vy);
    Mp     = iminterpolate(M,sx,sy);
    diff2  = (F-Mp).^2;
    area   = numel(M);
    
    % Three energy components
    e_sim  = sum(diff2(:)) / area;
    %e_dist = sum((cx(:)-sx(:)).^2 + (cy(:)-sy(:)).^2) / area;
    e_reg  = sum(sx(:).^2 + sy(:).^2) / area;
    
    % Total energy
    %e      = e_sim + (sigma_i^2/sigma_x^2) * e_reg;
    e = e_sim;

end

