% sregister.m
% Created: Dec 21st, 2011
%
% Spectral correspondence registration
% Finds correspondence using eigenmodes and updates the moving image
%
% F - Fixed image
% M - Moving image
% opt - options:
%    For registration:
%       niter            - number of iterations                                    (def: 5)
%       diffeomorphic    - diffeomorphic updates (0 or 1)                          (def: 1)
%       sigma_diffusion  - kernel width for diffusion                              (def: 1)
%       min_sigma_corr   - minimum width of diffusion kernel for spectral
%                          correspondence map                                      (def: 5)
%    For eigenmodes computation:
%       ne               - number of eigenmodes (including null vector)            (def: 3)
%       lambda_edges     - weight on graph edges (0..inf)                          (def: 0)
%       lambda_nodes     - weight on graph nodes (0..1)                            (def: 1)
%    For spectral correspondence:
%       null_weight      - weight of null eigenmode in spectral embedding          (def: 1)
%       eigenmode_weight - weight of one eigenmode in spectral embedding           (def: 1)
%       spatial_weight   - weight of spatial coordinates                           (def: 1)
%       image_weight     - weight of image intensities                             (def: 1)
%       bkg_weight       - weight of background components (null+eigenmodes)       (def: 0)
%
% Outputs:
%
% sx,sy - transformation so F(x+sx,y+sy) = M(x,y)
%

function [Mp, sx,sy, sx_inv,sy_inv, vx,vy, stats] = sym_sregister(F,M,opt)

    if nargin<3; opt = 0; end;

    if ~isfield(opt, 'niter');            opt.niter = 5;            end;
    if ~isfield(opt, 'sigma_diffusion');  opt.sigma_diffusion = 1;  end;
    if ~isfield(opt, 'min_sigma_corr');   opt.min_sigma_corr = 5;   end;
    if ~isfield(opt, 'sigma_i');          opt.sigma_i = 1;          end;
    if ~isfield(opt, 'sigma_x');          opt.sigma_x = 1;          end;
    if ~isfield(opt, 'ne');               opt.ne = 3;               end;
    if ~isfield(opt, 'lambda_edges');     opt.lambda_edges = 0;     end;
    if ~isfield(opt, 'lambda_nodes');     opt.lambda_nodes = 1;     end;
    if ~isfield(opt, 'null_weight');      opt.null_weight = 1;      end;
    if ~isfield(opt, 'eigenmode_weight'); opt.eigenmode_weight = 1; end;
    if ~isfield(opt, 'spatial_weight');   opt.spatial_weight = 1;   end;
    if ~isfield(opt, 'image_weight');     opt.image_weight = 1;     end;
    if ~isfield(opt, 'bkg_weight');       opt.bkg_weight = 0;       end;
    if ~isfield(opt, 'id_fixed');         opt.id_fixed = 'fixed';   end;
    if ~isfield(opt, 'id_moving');        opt.id_moving='id_moving';end;
    if ~isfield(opt, 'descend');          opt.descend = 0;          end;
    if ~isfield(opt, 'useminimum');       opt.useminimum = 1;       end;
    if ~isfield(opt, 'do_plotenergy');    opt.do_plotenergy = 0;    end;
    if ~isfield(opt, 'do_avi');           opt.do_avi = 0;           end;
    if ~isfield(opt, 'do_print');         opt.do_print = 0;         end;
    if ~isfield(opt, 'do_display');       opt.do_display = 1;       end;
    
    if opt.do_avi
        if ~isfield(opt, 'aviobj')
            opt.aviobj = avifile('spectral-demons.avi','compression','None', 'fps',10);
            opt.do_closeavi = 1; % create and close avi file here
        end
        if ~isfield(opt, 'do_closeavi'); opt.do_closeavi = 0; end;
        global aviobj;
        aviobj = opt.aviobj;
    end;
    
    %mode = 'nearest';
    mode = 'linear';

    disp(['Registering ...']);
    
    % Initial deformation is zero
    vx_fm = zeros(size(F)); % velocity of F toward M
    vy_fm = zeros(size(F));
    vx_mf = zeros(size(F)); % velocity of M toward F
    vy_mf = zeros(size(F));
    vx    = zeros(size(F)); % total velocity
    vy    = zeros(size(F));
    
    ne = opt.ne;
    if opt.null_weight==0;        ne = ne-1; end;
    if opt.eigenmode_weight==0;   ne = 0; end;
    
    % Build embeddings
    idx = 1:ne;
    [Bf,Ef,Vf] = buildembedding(F,opt);
    [Bm,Em,Vm] = buildembedding(M,opt);
    [Bm(:,:,idx),Vm(idx)] = reorderembedding(Bf(:,:,idx),Bm(:,:,idx),Vf(idx),Vm(idx),F,M);

    [Ef_before, Vf_before]  = spectrum_mask(F,  3, opt.lambda_edges, opt.lambda_nodes);
    [Em_before, Vm_before]  = spectrum_mask(M,  3, opt.lambda_edges, opt.lambda_nodes);
    [Em_before(:,:,2:end),Vm_before(2:end)] = reorderembedding(Ef_before(:,:,2:end),Em_before(:,:,2:end),Vf_before(2:end),Vm_before(2:end),F,M);
    
    if opt.do_display
        showiteration(F,M,Bf,Bm);
    end
    
    % Print embeddings
    if opt.do_print
        for iter_channel = 1:3
            Et = Ef_before(:,:,iter_channel);
            clf; imagesc(Et); daspect([1 1 1]); axis off; printfigure(['images/spectral-fixed-' opt.id_fixed '-moving-' opt.id_moving '-eigenmode-' num2str(iter_channel) '-fixed-before'],3,3);
        end
        for iter_channel = 1:3
            Et = Em_before(:,:,iter_channel);
            clf; imagesc(Et); daspect([1 1 1]); axis off; printfigure(['images/spectral-fixed-' opt.id_fixed '-moving-' opt.id_moving '-eigenmode-' num2str(iter_channel) '-moving-before'],3,3);
        end
    end
    
    e_min = energy(F,M,vx,vy,opt.sigma_i,opt.sigma_x);
    vx_min = vx; vy_min = vy;
    convergence_cnt = 0;
    disp(['Iteration: 0 - energy: ' num2str(e_min)]);
    %e_min = 1e+100; % initial energy, must not be set here because of partial volume effect during iterations (setting it here might produce the best energy since there is no partial volume)
    
    
    for iter = 1:opt.niter

        disp(['Iteration ' num2str(iter)]);
        
        % Transformations
        [sx_fm,sy_fm] = expfield(+vx,+vy); % +v is F to M
        [sx_mf,sy_mf] = expfield(-vx,-vy); % -v is M to F
        
        % Interpolated images
        Fp  = iminterpolate(F,  sx_mf,sy_mf, mode);
        Mp  = iminterpolate(M,  sx_fm,sy_fm, mode);
        %Fp(Fp<0) = 0; Fp(Fp>1) = 1; % valid intensities
        %Mp(Mp<0) = 0; Mp(Mp>1) = 1; % valid intensities


        %%% Spectral Correspondence
        
        % Build embeddings
        [Bfp,Efp,Vfp] = buildembedding(Fp,opt);
        [Bmp,Emp,Vmp] = buildembedding(Mp,opt);

        % change sign if necessary
        %Bmp = signembedding(Bf,Bmp);
        idx = 1:ne;
        Bfw = Bf; Bfpw = Bfp; Bmw = Bm; Bmpw = Bmp;
        if ne>0 && opt.eigenmode_weight>0
            [Bfp(:,:,idx),Vfp(idx)] = reorderembedding(Bf(:,:,idx),Bfp(:,:,idx),Vf(idx),Vfp(idx),F,Fp);
            [Bmp(:,:,idx),Vmp(idx)] = reorderembedding(Bf(:,:,idx),Bmp(:,:,idx),Vf(idx),Vmp(idx),F,Mp);

            [Bfw(:,:,idx),Bmpw(:,:,idx),scale] = weighteigenmodes(Bf (:,:,idx),Bmp(:,:,idx),Vf (idx),Vmp(idx));
            [Bmw(:,:,idx),Bfpw(:,:,idx),scale] = weighteigenmodes(Bm (:,:,idx),Bfp(:,:,idx),Vm (idx),Vfp(idx));
        else
            Bfw = Bfp;
            Bmw = Bmp;
        end

        if opt.do_print
            [sx_fm,sy_fm] = expfield(+vx,+vy);
            Mw  = iminterpolate(M,  sx_fm,sy_fm, mode);
            clf; imagesc(Fp+4*Mw); daspect([1 1 1]); axis off; printfigure(['images/spectral-fixed-' opt.id_fixed '-moving-' opt.id_moving '-spectral-iter-' num2str(iter)],3,3);
        end

        % Overlay both images
        if opt.do_display
            zr = zeros(size(F));
            diff = cat(3,F,zr,Mp);
            imshow(diff,'InitialMagnification','fit');
        end
        
        % Display embedding
        if opt.do_display
            %clf;imfuse(F,Mp);
            %showiteration(Fp,Mp,Bfpw,Bmpw);
            showiteration(Fp,Mp,Bfp,Bmp);
        end

        % Find symmetric correspondences
        idx = 1:size(Bmp,3);
        %[ux_fm,uy_fm] = findneighboringcorrespondence(Fp,Mp,Bfpw(:,:,idx),Bmpw(:,:,idx));
        %[ux_mf,uy_mf] = findneighboringcorrespondence(Mp,Fp,Bmpw(:,:,idx),Bfpw(:,:,idx));
        [ux_fm,uy_fm] = findneighboringcorrespondence(F,Mp,Bfw(:,:,idx),Bmpw(:,:,idx));
        [ux_mf,uy_mf] = findneighboringcorrespondence(M,Fp,Bmw(:,:,idx),Bfpw(:,:,idx));

        % Combine updates
        %ux = (F>0 & M>0).*(ux_fm-ux_mf)/2 + (F>0 & M<=0).*ux_fm - (F<=0 & M>0).*ux_mf;
        %uy = (F>0 & M>0).*(uy_fm-uy_mf)/2 + (F>0 & M<=0).*uy_fm - (F<=0 & M>0).*uy_mf;
        ux = (ux_fm-ux_mf)/2;
        uy = (uy_fm-uy_mf)/2;

        % Find smoothing
        norm            = sqrt(ux.^2 + uy.^2);
        %opt.sigma_fluid = max(norm(:))/3 /2;
        %opt.sigma_fluid = mean(norm(:)) + 5*std(norm(:));
        opt.sigma_fluid = mean([ mean(norm(:)) + 5*std(norm(:)) max(norm(:))/5 ]);
        %opt.sigma_fluid = mean(norm(norm>.2))/3; if numel(norm(norm>.2))==0; opt.sigma_fluid = 1; end;
        opt.sigma_fluid = max(opt.sigma_fluid, opt.min_sigma_corr);
        %opt.sigma_fluid = 0;

        % Fluid
        disp(['Sigma corr: ' num2str(opt.sigma_fluid)]);
        ux = imgaussian(ux,opt.sigma_fluid);
        uy = imgaussian(uy,opt.sigma_fluid);

        % Scale back update field
        if 0
            z   = imgaussian(1*(F>0 | M>0),opt.sigma_fluid);
            ux  = ux./(z + (z==0));
            uy  = uy./(z + (z==0));                
        end

        % Show update field
        if opt.do_display
            %[sx_fm,sy_fm] = expfield(vx,vy);
            %showupdatefield(sx_fm,sy_fm,3,0);
            [sx_fm,sy_fm] = expfield(ux,uy);
            showupdatefield(sx_fm,sy_fm,3,0);
            %[sx_mf,sy_mf] = expfield(-ux,-uy);
            %showupdatefield(sx_mf,sy_mf,3,0);
            drawnow;
        end
        
        %% Show figure with arrows (for eccv paper)
        if opt.do_display
            if 0%iter==1 || iter==2 || iter==5 || iter==15
                clf;imfuse(F,Mp); axis([15 82 20 87]);                          printfigure(['images/iter-' num2str(iter)],2,2,0);
                hold on; showvector(ux,uy,5,1); hold off; axis([15 82 20 87]);  printfigure(['images/iter-' num2str(iter) '-arrows'],2,2,1);
            end
        end
        

        % show current segmentations
        if 0
            %subplot(2,2,2); imagesc(Fp+2*(Bfp(:,:,2)<0)); daspect([1 1 1]);
            %subplot(2,2,4); imagesc(Mp+2*(Bmp(:,:,2)<0)); daspect([1 1 1]);
            subplot(4,4,13); imagesc(Fp+2*(Bfp(:,:,2)<0)); daspect([1 1 1]);
            subplot(4,4,14); imagesc(Mp+2*(Bmp(:,:,2)<0)); daspect([1 1 1]);
        end

        %%% Update fields
            
        % Updating velocity fields
        vx = vx + ux;
        vy = vy + uy;

        % Diffusion
        %opt.sigma_diffusion = 1;
        vx = imgaussian(vx,opt.sigma_diffusion);
        vy = imgaussian(vy,opt.sigma_diffusion);

        % Compute energy
        e      = energy(F,M,vx,vy,opt.sigma_i,opt.sigma_x);
        disp(['Iteration: ' num2str(iter) ' - ' 'energy: ' num2str(e)]);
        if e<e_min
            vx_min = vx; vy_min = vy;
            e_min  = e;
            convergence_cnt = 0;
        else
            if opt.descend;
                convergence_cnt = convergence_cnt + 1;
                if convergence_cnt>3
                    break;
                end
            end
        end
        stats.energy(iter) = e;
        if opt.do_plotenergy
            subplot(4,4,16);
            plot(1:iter,stats.energy,'r-'); xlim([0 opt.niter]);
            drawnow
        end

        if opt.do_avi
            [sx_fm,sy_fm] = expfield(+vx,+vy);
            Mp  = iminterpolate(M,  sx_fm,sy_fm, mode);

            gcf_old = gcf; figure(99);
            subplot(2,4,3); showimagesc(Mp,'+ SDemons (Moving)');
            
            for e=1:size(Bfp,3)
                if e>2; break; end;
                idx1 = [21 29]; 
                idx2 = [22 30]; 
                subplot(4,8,idx1(e)); showimagesc(Bfp(:,:,e),['Eigenmode ' num2str(e) ' (Fixed)'],'caxis',[-1 1]);
                subplot(4,8,idx2(e)); showimagesc(Bmp(:,:,e),['Eigenmode' num2str(e) ' (Moving)'],'caxis',[-1 1]);
            end

            %colormap gray; clf; subplot('position',[0 0 1 1]);
            %imagesc(Mp); daspect([1 1 1]); axis(ceil([26 153 26 153]/1)); axis off;
            %imagesc(Mp); daspect([1 1 1]); axis(ceil([42 42 46 46]/1)+ceil([0 36 0 36]/1)); axis off; % hat right
            %imagesc(Mp); daspect([1 1 1]); axis(ceil([102 102 58 58]/1)+ceil([0 36 0 36]/1)); axis off; % hat right
            drawnow
            
            for f=1:2; aviobj = addframe(aviobj,getframe(gcf)); end;
            figure(gcf_old);
        end;
        
    end % registration loop
    
    % Get best transformation
    if opt.useminimum
        vx = vx_min;
        vy = vy_min;
    end
    [sx,sy]         = expfield(+vx,+vy);
    [sx_inv,sy_inv] = expfield(-vx,-vy);
    
    % Compute final moving image
    [sx,sy]         = expfield(+vx,+vy);
    [sx_inv,sy_inv] = expfield(-vx,-vy);

    Mp  = iminterpolate(M,  sx,sy, mode);

    %%% Display segmentation
    
    if opt.do_display || opt.do_avi

        % Display final transform
        showimage(F,'Fixed', M,'Moving', Mp,'M Warped', 0.2*F+0.8*Mp,'Sum', 'nbrows',2, 'row',1);
        
        % Find Segmentation
        Ef = Ef_before; Vf = Vf_before;
        [Emp,Vmp] = spectrum_mask(Mp, 3, opt.lambda_edges, opt.lambda_nodes);
        [Emp(:,:,2:end),Vmp(2:end)] = reorderembedding(Ef(:,:,2:end),Emp(:,:,2:end),Vf(2:end),Vmp(2:end),F,Mp);

        % Left / Right
        LVf = Ef (:,:,2)<0;
        RVf = Ef (:,:,2)>0;
        LVm = Emp(:,:,2)<0;
        RVm = Emp(:,:,2)>0;

        % Antero / Posterio
        Af = Ef (:,:,3)<0;
        Pf = Ef (:,:,3)>0;
        Am = Emp(:,:,3)<0;
        Pm = Emp(:,:,3)>0;

        % Segmentations
        Sf = (2.5*LVf + .5*RVf + 1.5*Af + .5*Pf)/4;
        Sm = (2.5*LVm + .5*RVm + 1.5*Am + .5*Pm)/4;
        
        [x,y] = meshgrid(1:size(F,2),1:size(F,1));

        subplot(1,2,1);
        imagesc(Sf); daspect([1 1 1]); axis off; %axis(lim);
        hold on; contour(x,y,F,2, 'LineWidth',1,'Color','w'); hold off; 

        subplot(1,2,2);
        imagesc(Sm); daspect([1 1 1]); axis off; %axis(lim);
        hold on; contour(x,y,Mp,2, 'LineWidth',1,'Color','w'); hold off; 

        % Print embeddings
        if opt.do_print
            for iter_channel = 1:3
                Et = Ef(:,:,iter_channel);
                clf; imagesc(Et); daspect([1 1 1]); axis off; printfigure(['images/spectral-fixed-' opt.id_fixed '-moving-' opt.id_moving '-eigenmode-' num2str(iter_channel) '-fixed-after'],3,3);
            end
            for iter_channel = 1:3
                Et = Emp(:,:,iter_channel);
                clf; imagesc(Et); daspect([1 1 1]); axis off; printfigure(['images/spectral-fixed-' opt.id_fixed '-moving-' opt.id_moving '-eigenmode-' num2str(iter_channel) '-moving-after'],3,3);
            end
            clf; imagesc(.6*Sf+.4*F);  daspect([1 1 1]); axis off; printfigure(['images/spectral-fixed-' opt.id_fixed '-moving-' opt.id_moving '-fixed-segmentation'],3,3);
            clf; imagesc(.6*Sm+.4*Mp); daspect([1 1 1]); axis off; printfigure(['images/spectral-fixed-' opt.id_fixed '-moving-' opt.id_moving '-moving-segmentation'],3,3);
        end

        % Show segmentation
        showimage(.6*Sf+.4*F,'Fixed Segmentation', .6*Sm+.4*Mp,'Moving Segmentation', 'nbrows',2, 'row',2);
        drawnow;

        if opt.do_avi && opt.do_closeavi
            gcf_old = gcf; figure(99);
            aviobj = close(aviobj);
            figure(gcf_old);
        end
    end

end

% Smooth correspondence map
% Created: Jan. 7th, 2012 - Changed: Jan. 7th, 2012
%
function [cx,cy] = smoothcorrespondence(cx,cy, min_sigma_corr)

    if nargin<3; min_sigma_corr = 5; end;

    % Threshold large deformations
    th = 50; % maximal displacement allowed
    idx = abs(cx.^2 + cy.^2) > (th^2); % index to remove
    cx(idx) = 0;
    cy(idx) = 0;

    % Find smooth kernel width
    norm = sqrt(cx(:).^2 + cy(:).^2);
    s1 = max(norm(:))/3; % sigma to use is maximal vector magnitude (/3 sigmas)
    s2 = std(norm(:));   % if vector magnitude is very variable, smooth it
    s = min(s1,s2);
    s = max(min_sigma_corr,s); % minimal smoothness
    disp(['sigma: ' num2str(s)]);
    
    % Smooth update
    cx = imgaussian(cx,s);
    cy = imgaussian(cy,s);

end

% Build embedding
% Created: Dec 21st, 2011
%
function [B,E,V] = buildembedding(I,opt)

    s  = RandStream.create('mt19937ar','seed',1); RandStream.setDefaultStream(s); % always same random numbers
    
    % image limits
    lim = [1 size(I,2) 1 size(I,2)];
    
    %% 1 - Adding spectrum
    
    last_channel = 0;
    
    % Find spectrum
    if opt.eigenmode_weight>0
        [E,V] = spectrum_mask(I, opt.ne, opt.lambda_edges, opt.lambda_nodes, 0);
    else
        E = []; V = [];
    end
    
    % Adding null vector
    if opt.null_weight>0
        B(:,:,last_channel+1) = opt.null_weight * E(:,:,1);
        last_channel = last_channel + 1;
    end

    % Adding spectral coordinates
    if opt.eigenmode_weight>0
        B(:,:,(last_channel+1):(last_channel+opt.ne-1)) = opt.eigenmode_weight * E(:,:,2:opt.ne);
        last_channel = last_channel + opt.ne-1;
    end
    
    % Trimming eigenvectors if necessary
    if opt.null_weight==0 && opt.eigenmode_weight==0
        E = []; V = [];
    elseif opt.null_weight==0 && opt.eigenmode_weight>0
        E(:,:,1) = [];
        V(1) = [];
    elseif opt.null_weight>0 && opt.eigenmode_weight==0
        E(:,:,2:end) = []; V(2:end) = [];
    end
    
    % Adding intensities
    if opt.image_weight>0
        B(:,:,(last_channel+1)) = opt.image_weight * I;
        last_channel = last_channel + 1;
    end
    
    % Adding spatial coordinates (points remain close to each other
    if opt.spatial_weight>0
        ni = size(I,1); nj = size(I,2);
        [i,j] = ndgrid(1:ni,1:nj); i = 2*(i-1)/(ni-1)-1; j = 2*(j-1)/(nj-1)-1;
        B(:,:,(last_channel+1)) = opt.spatial_weight * i;
        B(:,:,(last_channel+2)) = opt.spatial_weight * j;
        last_channel = last_channel + 2;
    end
    
    %% 2 - Adding background spectrum

    if opt.bkg_weight>0

        % Inverse image (white becomes black, black becomes white)
        I_bkg = max(I(:)) - I;

        % Find spectrum
        [E_bkg,V_bkg] = spectrum_image(I_bkg, opt.ne, opt.lambda_edges, opt.lambda_nodes);
        E(:,:,(end+1):(end+opt.ne)) = E_bkg;
        V(:,  (end+1):(end+opt.ne)) = V_bkg;

        % Adding null vector
        if opt.null_weight>0
            B(:,:,(end+1)) = opt.bkg_weight * opt.null_weight * E_bkg(:,:,1);
        end

        % Adding spectral coordinates
        if opt.eigenmode_weight>0
            B(:,:,(end+1):(end+opt.ne-1)) = opt.bkg_weight * opt.eigenmode_weight * E_bkg(:,:,2:opt.ne);
        end

        % Adding intensities
        if opt.image_weight>0
            B(:,:,(end+1)) = opt.bkg_weight * opt.image_weight * I_bkg;
        end
    end

end

%% Weight eigenmodes with eigenvalues ratio
%  Created: Jan. 10th, 2012 - Changed: Jan. 10th, 2012
%
function [Ef,Em,scale] = weighteigenmodes(Ef,Em,Vf,Vm)

    % average eigenvalues
    V = mean([Vf Vm],2);

    % scale compared to the Fiedler value
    Vref = V(1);
    %if V(1) < 1e-6 && size(Vf,1)>1; Vref = V(2); end; % first non null eigenvalue
    
    % scaling factors for each vector
    scale = V ./ Vref;
    if scale(1)<1e-6; scale(1) = 1; end;
    %scale = (1./scale).^2;
    scale = sqrt(1./scale);
    %scale = 1./scale;
    
    % scale eigenvectors
    for i=1:size(Ef,3)
        Ef(:,:,i) = Ef(:,:,i) * scale(i);
        Em(:,:,i) = Em(:,:,i) * scale(i);
    end
    
end


%% Get spectrum of image
%  Changed: Dec 21st, 2011
%
function [E,V] = spectrum_image(I,ne,lambda_edge,lambda_node)

    piggybackscale = sqrt(2); % piggyback image before getting eigenmodes

    [I,lim] = piggyback(I,piggybackscale); % move boundary far away

    [E,V] = spectrum_mask(I,ne,lambda_edge,lambda_node);

    I = I(lim(1):lim(2),lim(3):lim(4));
    E = E(lim(1):lim(2),lim(3):lim(4),:);

    % Scale eigenvector to fit -1:+1
    for i=1:ne
        Ei      = E(:,:,i);

        E(:,:,i) = 2*(E(:,:,i)-min(Ei(:)))/range(Ei(:)) - 1; % fits range 0:+1 within mask
    end
    
end

%% Flip signs in embedding components
%  Created: Dec 21st, 2011
%
function B2 = signembedding(B1,B2)

    nchannels = size(B1,3);
    
    % Check if eigenvector signs match
    for i=1:nchannels
        b1 = B1(:,:,i);
        b2 = B2(:,:,i);

        b1 = 2*(b1-min(b1(:)))/range(b1(:))-1;
        b2 = 2*(b2-min(b2(:)))/range(b2(:))-1;

        diff1 = sum((b1(:)-b2(:)).^2);
        diff2 = sum((b1(:)+b2(:)).^2);

        if diff2<diff1; B2(:,:,i) = -B2(:,:,i); end;
    end

end

%% Show embeddings and images
%  Created: Dec 21st, 2011
%
function showiteration(F,Mp, Bf,Bmp)

    clf;
    
    lim = [1 size(F,2) 1 size(F,2)]; % limits of image
    
    % Show embeddings
    nbrows = size(Bf,3);
    for i=1:nbrows
        showimage(Bf(:,:,i),'Fixed', Bmp(:,:,i),'Moving', 'lim',lim, 'caxis',[-1 1], 'nbrows',nbrows, 'row',i, 'nbcols',4);
    end

    % Show deformed image
    subplot(1,2,2);
    imagesc(F+4*Mp,[0 5]);
    daspect([1 1 1]);
    axis off;
    
    drawnow
    
end

%% Show vector fields
%  Created: Dec 21st, 2011
%
function showupdatefield(ux,uy,downsample,scale)

    if nargin<3; downsample = 4; end;
    if nargin<4; scale      = 0; end;

    % Show vector field on images
    hold on; showvector(ux,uy,downsample,scale); hold off;
    
    % Smoothness of update field
    [gy,gx] = gradient(sqrt(ux.^2 + uy.^2));
    u = sum(sqrt(gx(:).^2 + gy(:).^2));
    disp(['smoothness: ' num2str(u)]);

end

%% Get energy
function e = energy(F,M,vx,vy,sigma_i,sigma_x,lim)

    if nargin<5; sigma_i = 1; end;
    if nargin<6; sigma_x = 1; end;
    if nargin<7; lim = [1 size(F,1) 1 size(F,2)]; end;
    
    % use real image size
    F = F(lim(1):lim(2),lim(3):lim(4));
    M = M(lim(1):lim(2),lim(3):lim(4));
    vx = vx(lim(1):lim(2),lim(3):lim(4));
    vy = vy(lim(1):lim(2),lim(3):lim(4));
        
    % Intensity difference
    [sx,sy] = expfield(vx,vy);
    Mp     = iminterpolate(M,sx,sy);
    diff2  = (F-Mp).^2;
    area   = numel(M);
    
    % Three energy components
    e_sim  = sum(diff2(:)) / area;
    %e_dist = sum((cx(:)-sx(:)).^2 + (cy(:)-sy(:)).^2) / area;
    e_reg  = sum(sx(:).^2 + sy(:).^2) / area;
    
    % Total energy
    %e      = e_sim + (sigma_i^2/sigma_x^2) * e_reg;
    e = e_sim;

end

